"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util");
var Schema = /** @class */ (function () {
    function Schema(props, state) {
        if (props === void 0) { props = {}; }
        if (state === void 0) { state = {}; }
        this.props = props;
        this.state = state;
    }
    Schema.prototype.setProps = function (props) {
        return new Schema(__assign({}, util_1.copyJson(this.props), props), __assign({}, this.state));
    };
    Schema.prototype.getState = function (key) {
        return this.state[key];
    };
    Schema.prototype.setState = function (key, val) {
        this.state[key] = val;
    };
    Schema.prototype.toJSON = function () {
        return util_1.copyJson(this.props);
    };
    /*======================
    GLOBAL
    ========================*/
    Schema.prototype.type = function (type) {
        return this.setProps({ type: type });
    };
    Schema.prototype.enum = function (values) {
        return this.setProps({ enum: values });
    };
    Schema.prototype.const = function (constValue) {
        return this.setProps({ const: constValue });
    };
    Schema.prototype.anyOf = function (schemas) {
        return this.setProps({ anyOf: schemas.map(function (s) { return s.props; }) });
    };
    Schema.prototype.oneOf = function (schemas) {
        return this.setProps({ oneOf: schemas.map(function (s) { return s.props; }) });
    };
    // TODO: these should be intersection types but that doesn't work :(
    Schema.prototype.allOf = function (schemas) {
        return this.setProps({ allOf: schemas.map(function (s) { return s.props; }) });
    };
    Schema.prototype.title = function (title) {
        return this.setProps({ title: title });
    };
    Schema.prototype.description = function (description) {
        return this.setProps({ description: description });
    };
    Schema.prototype.default = function (defaultValue) {
        return this.setProps({ default: defaultValue });
    };
    Schema.prototype.example = function (example) {
        return this.setProps({ example: example });
    };
    Schema.prototype.not = function (schema) {
        return this.setProps({ not: schema.props });
    };
    /*============
    Array
    =============*/
    Schema.prototype.items = function (items) {
        if (Array.isArray(items)) {
            return this.setProps({ items: items.map(function (i) { return i.props; }) });
        }
        else {
            return this.setProps({ items: items.props });
        }
    };
    Schema.prototype.additionalItems = function (additionalItems) {
        if (additionalItems === true) {
            return this.setProps({ additionalItems: true });
        }
        else if (additionalItems === false) {
            return this;
        }
        else {
            return this.setProps({ additionalItems: additionalItems.props });
        }
    };
    Schema.prototype.contains = function (contains) {
        return this.setProps({ contains: contains.props });
    };
    Schema.prototype.maxItems = function (maxItems) {
        return this.setProps({ maxItems: maxItems });
    };
    Schema.prototype.minItems = function (minItems) {
        return this.setProps({ minItems: minItems });
    };
    Schema.prototype.uniqueItems = function (uniqueItems) {
        return this.setProps({ uniqueItems: uniqueItems });
    };
    /*===============
    OBJECT
    =================*/
    Schema.prototype.properties = function (properties) {
        var props = Object.keys(properties).reduce(function (acc, key) {
            acc[key] = properties[key].props;
            return acc;
        }, {});
        return this.setProps({ properties: props });
    };
    Schema.prototype.required = function () {
        var required = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            required[_i] = arguments[_i];
        }
        return this.setProps({ required: required });
    };
    Schema.prototype.additionalProperties = function (additionalProperties) {
        if (additionalProperties instanceof Schema) {
            return this.setProps({ additionalProperties: additionalProperties.props });
        }
        else {
            return this.setProps({ additionalProperties: additionalProperties });
        }
    };
    Schema.prototype.patternProperties = function (patternProperties) {
        var props = Object.keys(patternProperties).reduce(function (acc, key) {
            acc[key] = patternProperties[key].props;
            return acc;
        }, {});
        return this.setProps({ patternProperties: props });
    };
    Schema.prototype.dependencies = function (dependencies) {
        var props = Object.keys(dependencies).reduce(function (acc, key) {
            var value = dependencies[key];
            if (Array.isArray(value)) {
                acc[key] = value;
            }
            else {
                acc[key] = value.props;
            }
            return acc;
        }, {});
        return this.setProps({ dependencies: props });
    };
    Schema.prototype.maxProperties = function (maxProperties) {
        return this.setProps({ maxProperties: maxProperties });
    };
    Schema.prototype.minProperties = function (minProperties) {
        return this.setProps({ minProperties: minProperties });
    };
    /*===================
    NUMBER
    =====================*/
    Schema.prototype.maximum = function (maximum) {
        return this.setProps({ maximum: maximum });
    };
    Schema.prototype.minimum = function (minimum) {
        return this.setProps({ minimum: minimum });
    };
    Schema.prototype.exclusiveMaximum = function (exclusiveMaximum) {
        return this.setProps({ exclusiveMaximum: exclusiveMaximum });
    };
    Schema.prototype.exclusiveMinimum = function (exclusiveMinimum) {
        return this.setProps({ exclusiveMinimum: exclusiveMinimum });
    };
    Schema.prototype.multipleOf = function (multipleOf) {
        return this.setProps({ multipleOf: multipleOf });
    };
    /*=============
    STRING
    ===============*/
    Schema.prototype.maxLength = function (maxLength) {
        return this.setProps({ maxLength: maxLength });
    };
    Schema.prototype.minLength = function (minLength) {
        return this.setProps({ minLength: minLength });
    };
    Schema.prototype.pattern = function (pattern) {
        return this.setProps({ pattern: pattern.source });
    };
    Schema.prototype.format = function (format) {
        return this.setProps({ format: format });
    };
    return Schema;
}());
exports.Schema = Schema;
